/*
 * Copyright (c) 2009-2011 Emmanuel Vadot <elbarto@homebrewdev.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* $Id: joy.h,v 1.11 2011/01/25 16:21:30 elbarto Exp $ */

/*! \file joy.h
    \brief Joystick related functions
 */

/*! \def JOY_1
    \brief The player 1 joystick
 */

#define JOY_1		0x0

/*! \def JOY_2
    \brief The player 2 joystick
 */

#define	JOY_2		0x1

/*! \def JOY_3
    \brief The player 3 joystick (with the ea4way)
 */

#define	JOY_3		0x2

/*! \def JOY_4
    \brief The player 4 joystick (with the ea4way)
 */

#define	JOY_4		0x3

/*! \def JOY_EXT
    \brief The ext joystick
 */

#define	JOY_EXT		0x2


/*! \def JOY_UP
    \brief The Up arrow
 */

#define JOY_UP		0x1

/*! \def JOY_DOWN
    \brief The Down arrow
 */
#define JOY_DOWN	0x2

/*! \def JOY_LEFT
    \brief The Left arrow
 */
#define JOY_LEFT	0x004

/*! \def JOY_RIGHT
    \brief The Right arrow
 */
#define JOY_RIGHT	0x008

/*! \def JOY_B
    \brief The B button
 */

#define JOY_B		0x010

/*! \def JOY_C
    \brief The C button
 */

#define JOY_C		0x020

/*! \def JOY_A
    \brief The A button
 */

#define JOY_A		0x040

/*! \def JOY_START
    \brief The START button
 */

#define JOY_START	0x080


/*! \def JOY_X
    \brief The X button
 */

#define JOY_X		0x400

/*! \def JOY_Y
    \brief The Y button
 */

#define	JOY_Y		0x200

/*! \def JOY_Z
    \brief The Z button
 */

#define JOY_Z		0x100

/*! \def JOY_MODE
    \brief The MODE button
 */

#define JOY_MODE	0x800

/*! \def JOY_IS_PRESSED(x, y)
    \brief Check if the button is pressed
 */

#define JOY_IS_PRESSED(x, y)	((x & y) != 0)

/*! \fn void joy_init(void)
    \brief initialize the joysticks

    To call before anything else
*/

void	joy_init(void);

/*! \def JOY_NONE
    \brief Means that no joypad is connected
*/

#define	JOY_NONE	0x0

/*! \def JOY_3BTS
    \brief Standard 3 buttons joypad
*/

#define	JOY_3BTS	0x1

/*! \def JOY_6BTS
    \brief Standard B buttons joypad
*/

#define	JOY_6BTS	0x2

/*! \fn u8 joy_gettype(u8 joy_num)
    \brief Return the type of the joypad (3 buttons or 6 buttons)
    \param joy_num The joystick number to read
*/

u8	joy_gettype(u8 joy_num);

/*! \fn u16 joy_read(u8 joy_num, u8 type)
    \brief Read the input for the joystick
    \param joy_num The joystick number to read
    \param type The type of joystick to read
*/

u16	joy_read(u8 joy_num, u8 type);

/*! \fn void joy_ea4way_set(u8 joy_num)
    \brief Set the joystick number to read for the ea4way
    \param joy_num The joystick number to read
*/

void	joy_ea4way_set(u8 joy_num);

/*! \fn u8 joy_isea4way(void)
    \brief Test is an ea4way is connected

    Return 1 if true, else 0
*/

u8	joy_isea4way(void);
