/*
 * Copyright (c) 2009-2011 Emmanuel Vadot <elbarto@homebrewdev.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* $Id: vdp.s,v 1.14 2011/01/26 04:52:36 elbarto Exp $ */
	
.global vdp_setreg
.global	vdp_pointto
.global vdp_write
.global vdp_write16
.global vdp_write32
.global	vdp_writetocram
.global	vdp_writetovram
.global	vdp_writetovsram
.global vdp_dma
.global vdp_dmafill
.global	vdp_waitdma
.global vdp_clrvram
.global vdp_clrvsram
.global vdp_waitvsync

VDP_DATA =		0x00c00000
VDP_CTRL =		0x00c00004
VDP_HVCOUNTER =		0x00c00008

vdp_setreg:
	movem.l	4(sp), d0-d1

	lsl.w	#8, d0
	or.w	#0x8000, d0
	or.w	d1, d0

	move.w	d0, (VDP_CTRL)
	rts

vdp_pointto:
	movem.l	4(sp), d0-d1

	and.w	#0x3FFF, d0
	and.w	#3, d1
	lsl.w	#8, d1
	lsl.w	#6, d1
	or.w	d1, d0
	move.w	d0, (VDP_CTRL)

	movem.l	4(sp), d0-d1
	lsr.w	#8, d0
	lsr.w	#6, d0
	and.w	#0x3C, d1
	lsl.w	#2, d1
	or.w	d1, d0
	move.w	d0, (VDP_CTRL)

	rts

vdp_write:
vdp_write16:
	movem.l	4(sp), d0
	move.w	d0, (VDP_DATA)
	rts

vdp_write32:
	movem.l	4(sp), d0
	move.l	d0, (VDP_DATA)
	rts

vdp_writetocram:
	movem.l	4(sp), d1/a0
	move.l	12(sp), d0

	or.w	#0xc000, d0

	move.w	d0, (VDP_CTRL)
	move.w	#0, (VDP_CTRL)

_writetocram:
	move.w	(a0)+, (VDP_DATA)
	dbra	d1, _writetocram
	rts

vdp_writetovram:
	/* Write destination address (1st cmd) */
	move.l	12(sp), d0
	and.w	#0x3FFF, d0
	or.w	#0x4000, d0
	move.w	d0, (VDP_CTRL)

	/* 2nd cmd */
	move.l	12(sp), d0
	lsr.w	#8, d0
	lsr.w	#6, d0
	move.w	d0, (VDP_CTRL)

	movem.l	4(sp), d0/a0
	divu.w	#2, d0
	subq.w	#1, d0
_writetovram:
	move.w	(a0)+, (VDP_DATA)
	dbra	d0, _writetovram
	rts

vdp_writetovsram:
	/* Write destination address (1st cmd) */
	move.l	12(sp), d0
	and.w	#0x3FFF, d0
	or.w	#0x4000, d0
	move.w	d0, (VDP_CTRL)

	/* 2nd cmd */
	move.l	12(sp), d0
	lsr.w	#8, d0
	lsr.w	#6, d0
	or.w	#0x10, d0
	move.w	d0, (VDP_CTRL)

	movem.l	4(sp), d0/a0
	divu.w	#2, d0
	subq.w	#1, d0
_writetovsram:
	move.w	(a0)+, (VDP_DATA)
	dbra	d0, _writetovsram
	rts

vdp_dma:
	move.l	4(sp), d0
	/* Setup DMA length */
	lsr.w	#1, d0
	and.w	#0xff, d0
	or.w	#0x9300, d0
	move.w	d0, (VDP_CTRL)
	move.l	4(sp), d0
	lsr.w	#8, d0
	lsr.w	#1, d0
	and.w	#0xff, d0
	or.w	#0x9400, d0
	move.w	d0, (VDP_CTRL)

	/* Setup DMA source address */
	/* Low */
	move.l	8(sp), d0
	lsr.l	#1, d0
	and.l	#0xff, d0
	or.l	#0x9500, d0
	move.w	d0, (VDP_CTRL)
	/* Mid */
	move.l	8(sp), d0
	lsr.l	#8, d0
	lsr.l	#1, d0
	and.l	#0xff, d0
	or.l	#0x9600, d0
	move.w	d0, (VDP_CTRL)
	/* High */
	move.l	8(sp), d0
	lsr.l	#8, d0
	lsr.l	#8, d0
	lsr.l	#1, d0
	and.l	#0xff, d0
	or.l	#0x9700, d0
	move.w	d0, (VDP_CTRL)

	/* Setup DMA dest address */
	/* 1st cmd */
	move.l	12(sp), d0
	move.l	16(sp), d1
	and.w	#0x3FFF, d0
	lsl.w	#8, d1
	lsl.w	#6, d1
	or.w	d1, d0
	move.w	d0, (VDP_CTRL)
	/* 2nd cmd */
	move.l	12(sp), d0
	move.l	16(sp), d1
	lsr.w	#8, d0
	lsr.w	#6, d0
	and.w	#0x4, d1
	lsl.w	#2, d1
	or.w	#0x80, d1
	or.w	d1, d0
	/* push to 2nd cmd to ram */
	move.w	d0, -(sp)
	move.w	(sp), (VDP_CTRL)
	move.w	(sp)+, d0
	rts

vdp_dmafill:
	/* Set autoinc to 1 */
	move.w	#0x8F01, (VDP_CTRL)
	
	/* Setup DMA length */
	move.l	4(sp), d0
	and.w	#0xff, d0
	or.w	#0x9300, d0
	move.w	d0, (VDP_CTRL)
	move.l	4(sp), d0
	and.w	#0xff00, d0
	lsr.w	#8, d0
	or.w	#0x9400, d0
	move.w	d0, (VDP_CTRL)

	/* Set DMA fill mode */
	move.w	#0x9780, (VDP_CTRL)
	
	/* Setup DMA dest address */
	/* 1st cmd */
	move.l	12(sp), d0
	and.w	#0x3FFF, d0
	or.w	#0x4000, d0
	move.w	d0, (VDP_CTRL)
	/* 2nd cmd */
	move.l	12(sp), d0
	lsr.w	#8, d0
	lsr.w	#6, d0
	or.w	#0x80, d0
	/* push to 2nd cmd to ram */
	move.w	d0, -(sp)
	move.w	(sp), (VDP_CTRL)
	move.w	(sp)+, d0

	/* Write fill data to vdp data port */
	move.w	8(sp), d0
	move.w	d0, (VDP_DATA)
	
	rts

vdp_waitdma:
	move.w	(VDP_CTRL), d0
_waitdma:
	btst	#1, d0
	bne.s	_waitdma
	rts

vdp_clrvram:
	move.l	#0x60000000, (VDP_CTRL)
	move.w	#0x8f02,(VDP_CTRL)
	move.w	#0x8000,d0
_clrvram:
	move.w	#0, (VDP_DATA)
	dbra	d0, _clrvram
	rts

vdp_clrvsram:
	move.l	#0x40000010, (VDP_CTRL)
	move.w	#0x8f02,(VDP_CTRL)
	move.w	#40,d0
_clrvsram:
	move.w	#0, (VDP_DATA)
	dbra	d0, _clrvsram
	rts

vdp_waitvsync:
	move.w	(VDP_CTRL), d0
	btst #3,d0
	bne.s vdp_waitvsync
_waitvsync:
	move.w (VDP_CTRL),d0
	btst #3,d0
	beq.s _waitvsync

	rts
